@@ -49,6 +49,28 @@
 #include "drmtest.h"
 #include "igt_aux.h"
 #include "ioctl_wrappers.h"
+/**
+ * TEST: dumb buffer
+ * Description: This is a test for the generic dumb buffer interface.
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: uapi
+ * Functionality: ioctl
+ * Feature: prime
+ * Test category: Compositor
+ *
+ * SUBTEST: create-clear
+ *
+ * SUBTEST: create-valid-dumb
+ *
+ * SUBTEST: invalid-bpp
+ *
+ * SUBTEST: map-invalid-size
+ *
+ * SUBTEST: map-uaf
+ *
+ * SUBTEST: map-valid
+ */
 
 IGT_TEST_DESCRIPTION("This is a test for the generic dumb buffer interface.");
 
@@ -121,7 +143,7 @@ static void invalid_dimensions_test(int fd)
 	memset(&create, 0, sizeof(create));
 	create.width = 4032;
 	create.height = 2016;
-	create.bpp = 24;
+	create.bpp = 0;
 	igt_assert_eq(__dumb_create(fd, &create), -EINVAL);
 
 	create.bpp = 32;
@@ -394,6 +416,6 @@ igt_main
 		always_clear(fd, 30);
 
 	igt_fixture {
-		close(fd);
+		drm_close_driver(fd);
 	}
 }
