@@ -22,7 +22,23 @@
  *
  */
 
+/**
+ * TEST: kms 3d
+ * Category: Display
+ * Description: Tests 3D mode setting.
+ */
+
 #include "igt.h"
+#include "xe/xe_query.h"
+
+/**
+ * SUBTEST:
+ * Description: Tests 3D mode setting.
+ * Driver requirement: i915, xe
+ * Functionality: stereoscopic
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
 
 IGT_TEST_DESCRIPTION("Tests 3D mode setting.");
 
@@ -111,5 +127,6 @@ igt_simple_main
 	kmstest_force_edid(drm_fd, connector, NULL);
 
 	drmModeFreeConnector(connector);
-	close(drm_fd);
+
+	drm_close_driver(drm_fd);
 }
